# This lets you install ShibbolethLogin through Plone, if you're into that.
# If you aren't using Plone, it doesn't hurt anything.

import os

from Globals import package_home
from Products.CMFFormController.ControllerPageTemplate import ControllerPageTemplate
from Products.CMFCore.utils import getToolByName
from HTMLParser import HTMLParser
from Products.PluggableAuthService.interfaces.plugins import IChallengePlugin
from Products.PluggableAuthService.PluggableAuthService import logger
from Products.PythonScripts.PythonScript import PythonScript
from zExceptions import BadRequest
try:
	from ShibbolethLogin.redirector import ShibbolethLoginHandler
except ImportError:
	from Products.ShibbolethLogin.redirector import ShibbolethLoginHandler
try:
	from ShibbolethLogin import shib_globals
except ImportError:
	from Products.ShibbolethLogin import shib_globals

def _firstIdOfClass(container, class_):
	"""Return the id of the first object of class `class_` within `container`.
	If there is none, return None."""
	for id in container.objectIds():
		if isinstance(container[id], class_):
			return id
	return None

def install(portal, reinstall=False):
	acl_users = getToolByName(portal, 'acl_users')

	# Put a ShibbolethLogin in the acl_users folder, if there isn't one:
	pluginId = _firstIdOfClass(acl_users, ShibbolethLoginHandler)
	if not pluginId:
		pluginId = 'ShibbolethLogin'
		constructors = acl_users.manage_addProduct[pluginId]
		constructors.manage_addShibbolethLogin(pluginId, title='ShibbolethLogin Plugin')

	extensionsPath = os.path.join(package_home(shib_globals), 'Extensions')

	# Add the login_form and logout scripts
	portal_skins = getToolByName(portal, 'portal_skins')
	try:
		form = getattr(portal_skins.plone_forms, 'login_form').document_src()
		if (portal_skins.Control_Panel.Products.CMFPlone.version.startswith('2')):
			zpt = open(os.path.join(extensionsPath, 'login_form2.zpt'))
			sInsertBefore = 'description_no_account'
		else:
			zpt = open(os.path.join(extensionsPath, 'login_form3.zpt'))
			sInsertBefore = 'login_form'
		shibzpt = zpt.readlines()
		zpt.close()
		parser = MyHTMLParser(sInsertBefore, "".join(shibzpt))
		parser.feed(form)
		parser.close()
		form = parser.getResult()
		login_form = ControllerPageTemplate('login_form', form)
		portal_skins.custom._setObject('login_form', login_form)

		pfc = getToolByName(portal, 'portal_form_controller')
		pfc.addFormValidators('login_form', '', '', 'login_form_validate')
		pfc.addFormAction('login_form', 'enable_cookies', '', '',
						  'traverse_to', 'string:enable_cookies')
		pfc.addFormAction('login_form', 'failure', '', '',
						  'redirect_to', 'string:login_form')
		pfc.addFormAction('login_form', 'failure_page', '', '',
						  'traverse_to', 'string:login_failed')
		pfc.addFormAction('login_form', 'success', '', '',
						  'traverse_to', 'string:logged_in')
	except Exception, err:
		print err
		logger.info(str(err))

	try:
		logout = PythonScript('logout')
		py = open(os.path.join(extensionsPath, 'logout_form.py'))
		logout.write("".join(py.readlines()))
		py.close()
		portal_skins.custom._setObject('logout', logout)
	except Exception, err:
		logger.info(str(err))

	# Now restore the configuration
	if reinstall:
		import pickle
		plugin = getattr(acl_users.plugins, pluginId)
		# Get the configuration out of the property, and delete the property.
		try:
			prop = "\n".join(acl_users.getProperty('sl_config'))
			#logger.info("sl_config = %s" % repr(prop))
			config = pickle.loads(prop)
		except Exception, err:
			logger.info("error getting config: %s of %r" % (str(err), repr(err)))
		for prop in plugin.propertyMap():
			if config.has_key(prop['id']):
				try:
					val = config[prop['id']]['value']
					if prop['type'] == 'lines':
						val = "\n".join(val)
					#logger.info("setting %s to %s" % (prop['id'], repr(val)))
					if config[prop['id']]['type'] == prop['type']:
						plugin.manage_changeProperties({prop['id']: val})
					elif prop['type'] == 'int':
						try:
							plugin.manage_changeProperties({prop['id']: int(val)})
						except TypeError:
							pass
					else:
						plugin.manage_changeProperties({prop['id']: str(val)})
				except Exception, err:
					logger.info("error in install: %s" % str(err))
	try:
		acl_users.manage_delProperties(['sl_config'])
	except:
		pass

def uninstall(portal, reinstall=False):
	acl_users = getToolByName(portal, 'acl_users')
	pluginId = _firstIdOfClass(acl_users, ShibbolethLoginHandler)
	if pluginId:
		if reinstall:
			import pickle
			plugin = getattr(acl_users.plugins, pluginId)
			config = {}
			for prop in plugin.propertyMap():
				config[prop['id']] = prop
				config[prop['id']]['value'] = plugin.getProperty(prop['id'])
			conf = pickle.dumps(config)
			acl_users.manage_addProperty(id='sl_config', type='lines', value=conf)
		acl_users.manage_delObjects(ids=[pluginId])  # implicitly deactivates
		portal_skins = getToolByName(portal, 'portal_skins')
		for ii in ('login_form', 'logout'):
			try:
				portal_skins.custom.manage_delObjects([ii])
			except Exception, err:
				logger.info(str(err))

class MyHTMLParser(HTMLParser): #IGNORE:R0904
	"""Pass all through, adding the given form, when the given pattern matches.

	In Plone 2.5, find 'heading_advanced_settings' and insert the Shib form."""
	from htmlentitydefs import entitydefs

	tSingles = ('br', 'img', 'input', 'hr')

	def __init__(self, sPattern, sForm):
		"""Set up output holders and state variables, and initialize parser."""
		self.lOutput = list()
		self._initVars()
		HTMLParser.__init__(self)
		self.sPattern = sPattern
		self.sForm = sForm

	def _initVars(self):
		"""Set/reset variables used to track mailto URLs."""
		self.bEmail = False
		self.sEncoded = ''
		self.sVisible = ''
		self.sAttrs = ''

	def handle_starttag(self, sTag, lAttrs):
		"""Pass through all start tags, except anchors of mailto urls."""
		lTemp = list()
		sMailto = ''
		for sKey, sVal in lAttrs:
			if sVal == self.sPattern:
				self.lOutput.append(self.sForm)
			lTemp.append('%s="%s"' % (sKey, sVal))
		if len(lTemp) == 0:
			if sTag in (self.tSingles):
				self.lOutput.append("<%s />" % sTag)
			else:
				self.lOutput.append("<%s>" % (sTag))
		else:
			if sTag in (self.tSingles):
				self.lOutput.append("<%s %s />" % (sTag, ' '.join(lTemp)))
			else:
				self.lOutput.append("<%s %s>" % (sTag, ' '.join(lTemp)))

	def handle_endtag(self, sTag):
		"Record when exiting a sought after tag."
		if sTag in self.tSingles:
			return
		self.lOutput.append("</%s>" % (sTag))

	def handle_data(self, sData):
		"""Save the data found."""
		self.lOutput.append(sData)

	def handle_charref(self, sData):
		"""Pass character references through."""
		self.lOutput.append('&#%s;' % sData)

	def handle_entityref(self, sData):
		"""Pass entity references through."""
		if self.entitydefs.has_key(sData):
			self.lOutput.append("&%s;" % (sData))
		else:
			# From Plone: this breaks non-standard entities that end with ';'
			self.lOutput.append("&%s" % (sData))

	def handle_comment(self, sData):
		"""Pass comments through."""
		self.lOutput.append("<!--%s-->" % sData)

	def handle_decl(self, sData):
		"""Pass declarations through."""
		self.lOutput.append("<!%s>" % sData)

	def getResult(self):
		"""Return any available HTML."""
		sTemp = ''.join(self.lOutput)
		#Remove the
		self.lOutput = list()
		return sTemp
