from Products.PluggableAuthService.interfaces.authservice import IPluggableAuthService
try:
    from ShibbolethLogin.redirector import ShibbolethLoginHandler
except ImportError:
    from Products.ShibbolethLogin.redirector import ShibbolethLoginHandler

def _firstIdOfClass(container, class_):
    """Return the id of the first object of class `class_` within `container`.
    If there is none, return None."""
    for id in container.objectIds():
        if isinstance(container[id], class_):
            return id
    return None

def addShibbolethLogin(context):
    """Find the nearest acl_users and adds and activates a ShibbolethLogin.

    Return a 1-tuple with the new ShibbolethLogin as its only element."""

    acl_users = getattr(context, 'acl_users', None)
    if acl_users is None:
        raise LookupError("No acl_users can be acquired or otherwise found.")

    pas = IPluggableAuthService(acl_users, None)
    if pas is None:
        raise ValueError("The nearest acl_users object is not a PluggableAuthService.")

    pluginId = _firstIdOfClass(acl_users, ShibbolethLoginHandler)
    if not pluginId:
        pluginId = 'ShibbolethLogin'
        setup = acl_users.manage_addProduct[pluginId]
        setup.manage_addShibbolethLogin(pluginId, 'ShibbolethLogin Plugin')

    return pas[pluginId]
